<?php

/** 
 * ejournal_journal_edit_form: return a form for editing a journal
 *
 * @param $edit
 *   an array of values handled by the form
 *
 * @return
 *   array of form elements 
 */
function ejournal_journal_edit_form($edit = array ()) {
	if (!$edit) {
		$edit['jid'] = 0;
	}
	$journal = ejournal_journal_get($edit['jid']);

	// Enabled?
	$enabled = array (
		'1' => 'published',
		'0' => 'not-published'
	);
	$form['enabled'] = array (
		'#type' => 'select',
		'#title' => t("Status"), 
    '#default_value' => $edit['enabled'], '#options' => $enabled, 
    '#description' => t("You may publish/close the journal with one click"),
    );

	// Journal identification  
	$form['title'] = array (
		'#type' => 'textfield',
		'#title' => t('Journal title'), 
    '#default_value' => $edit['title'], '#size' => 50, '#maxlength' => 128, 
    '#description' => t('The title for this journal.'), 
    '#required' => 1,
    );

	$form['path'] = array (
		'#type' => 'textfield',
		'#title' => t('Journal path'), 
    '#default_value' => $edit['path'], '#size' => 50, '#maxlength' => 64, 
    '#description' => t('The relative url where the journal will be accessible (e.g. ikaros - the journal will be at http://www.site.org/ikaros). Please avoid using spaces and accented characters.'), 
    '#required' => 1,
    );

	$form['description'] = array (
		'#type' => 'textarea',
		'#title' => t('Description'), 
    '#default_value' => $edit['description'], 
    '#cols' => 70, 
    '#rows' => 4, 
    '#description' => t('A description of this journal.'),
    );

	// ISSN  
	$form['issn'] = array (
		'#type' => 'textfield',
		'#title' => t('ISSN'), 
    '#default_value' => $edit['issn'], '#size' => 50, '#maxlength' => 64, 
    '#description' => t('The ISSN (International Standard Serial Number) is an eight-digit number which identifies periodical publications as such, including electronic serials. A number can be obtained from the <a href="http://www.issn.org">ISSN web site</a>.'), 
    '#required' => 0,
    );

	// Publisher
	$form['publisher_name'] = array (
		'#type' => 'textfield',
		'#title' => t('Publisher'), 
    '#default_value' => $edit['publisher_name'], '#size' => 50, '#maxlength' => 128, 
    '#description' => t('A person or institution responsible for the journal'),
    );

	$form['publisher_url'] = array (
		'#type' => 'textfield',
		'#title' => t('Publisher URL'), 
    '#default_value' => $edit['publisher_url'], '#size' => 50, '#maxlength' => 128, 
    '#description' => t('URL address of the publisher - fully qualified, e.g. http://www.mysite.org'),
    );

	$form['publisher_email'] = array (
		'#type' => 'textfield',
		'#title' => t('Publisher email'), 
    '#default_value' => $edit['publisher_email'], '#size' => 50, '#maxlength' => 64, 
    '#description' => t('Contact email for the publisher'),
    );

	// Identification of issues, group of items
	$form['issue'] = array (
		'#type' => 'fieldset',
		'#title' => t('Starting point and frequency'), 
    '#description' => t('Leave items blank if not used with this journal, chief editors can re-set numbers in the publishing process.'), 
    '#collapsible' => FALSE, '#collapsed' => FALSE);

	$options = _ejournal_journal_get_identification_formats();
	$form['issue']['issue_format'] = array (
		'#type' => 'radios',
		'#title' => t('Issue identification'), 
    '#default_value' => $edit['issue_format'], 
    '#options' => $options, 
    '#description' => t('Select the default format for identifying journal issues and/or volumes. It is believed that the items will be only numerical, but you may also choose the last option and write what you like to identify the issue (but ejournal module will not check numbering of issues for you).'), 
    '#required' => 1,
    );

	// this is saved in {ejournal_store}
	$form['issue']['issues_per_volume'] = array (
		'#type' => 'textfield',
		'#title' => t('Issues per volume'), 
    '#default_value' => $edit['issues_per_volume'], 
    '#size' => 50, 
    '#maxlength' => 64,
    );
	$form['issue']['volumes_per_year'] = array (
		'#type' => 'textfield',
		'#title' => t('Volumes per year'), 
    '#default_value' => $edit['volumes_per_year'], 
    '#size' => 50, '#maxlength' => 64,
    );

	// archive
	$order = array (
		'DESC' => 'descending',
		'ASC' => 'ascending'
	);
	$form['archive_order'] = array (
		'#type' => 'select',
		'#title' => t('Display order for listing issues in the archive'), 
    '#default_value' => $edit['archive_order'], 
    '#options' => $order, 
    '#description' => t('Descending order means that the most recent issues will appear at the top of the list.'),
    );

	// about page
	$form['about_page'] = array (
		'#type' => 'textfield',
		'#title' => t('About page'), 
    '#default_value' => $edit['about_page'], 
    '#size' => 50, '#maxlength' => 64, 
    '#description' => t('Insert a node number for &quot;about page&quot; of this journal. If none is given, the default archive will be displayed under /ejournal/about/{journal_number}'),
    );

	// Select a layout
	if (variable_get('ejournal_custom_layouts', '0')) {
		$layouts = ejournal_layouts();
		$form['ejournal_layout'] = array (
			'#type' => 'select',
			'#title' => t('Custom ejournal layout'), 
      '#default_value' => $edit['ejournal_layout'], 
      '#options' => $layouts, 
      '#description' => t('Select a layout to control the display of this journal. The custom layout should be placed inside of the ejournal folder, the file has filaname of theme_{name}.inc - the custom layout does not need to be complete file with all the ejournal theme functions. If the custom theme is not found, the default will be used instead.'),
      );
	}

	// Select vocabularies
	if (module_exists('taxonomy')) {

		$vids = ejournal_get_all_vocabularies();
		$selected_vids = $journal->vid ? split(',', ($journal->vid)) : '';

		$form['taxonomy'] = array (
			'#type' => 'fieldset',
			'#title' => t('Taxonomy'), 
      '#collapsible' => FALSE, 
      '#collapsed' => FALSE
      );
		$form['taxonomy']['vid'] = array (
			'#type' => 'select',
			'#title' => t('Section'), 
      '#default_value' => $selected_vids, 
      '#options' => $vids, 
      '#multiple' => TRUE, 
      '#description' => t("Select a vocabulary that contains sections/topics for this !journal. If none is selected, the journal can not be organized into sections.", array (	'!settings' => l(t('journal'	), 'ejournal/edit/journal/' . $edit['jid']))) . '<br/><b>' . t("WARNING: if you select vocabulary (or vocabularies) for section, your nodetypes must use them in order to be shown in the e-journal. If you select wrong vocabulary, your articles will not be displayed in the issue. (Note: you may select several vocabularies, and your nodetypes need not belong to all of them)") . '</b>',
      );

		$selected_vids = $journal->other_vid ? explode(',', $journal->other_vid) : '';
		$form['taxonomy']['other_vid'] = array (
			'#type' => 'select',
			'#title' => t('Other vocabularies'), 
      '#default_value' => $selected_vids, 
      '#options' => $vids, 
      '#description' => t("Apart from sections, you may want to choose other vocabularies which should be offered as index (i.e. list of taxonomy terms for browsing)."), '#multiple' => TRUE,
      );

	} else {
		drupal_set_message(t("Please, enable taxonomy module if you want to have your journal organized into sections/topics."));
	}

	// list of node-types associated with this journal, saved in {ejournal_store}  
	$registered = db_query("SELECT jid, jvalue AS type FROM {ejournal_store} WHERE jkey='node_types'");
	while ($ntype = db_fetch_object($registered)) {
		if ($ntype->jid != $edit['jid']) {
			$others[$ntype->type] = node_get_types('name', $ntype);
		}
	}
  
	foreach (node_get_types() as $type => $obj) {
		if (!$others[$type]) {
			$nodetypes[$type] = $obj->name;
		}
	}

	//node types considered for e-journal
	$form['doctypes'] = array (
		'#type' => 'fieldset',
		'#title' => t('Automatic selection rules'), 
    '#description' => t('Ejournal module &quot;gathers&quot; only specific node types. Here you can choose which node types will belong to this journal (other node types will simply be ignored). Node types already registered with another journal can not be selected again.'), 
    '#collapsible' => FALSE, 
    '#collapsed' => FALSE
    );

	$form['doctypes']['node_types'] = array (
		'#type' => 'checkboxes',
		'#title' => t('Content types'), 
    '#default_value' => explode(',', $edit['node_types']), 
    '#options' => $nodetypes, 
    '#description' => t("A list of node types you want to include in this journal."), 
    '#required' => 1,
    );

	if ($others) {
		$form['doctypes'][''] = array (
			'#type' => 'item',
			'#value' => t('Some node type(s) were already registered with another journal: %types',
			array (
				'%types' => implode(', ',
				$others
			)
		)),);
	}

	if (function_exists('queue_overview')) {
		$options = array (
    '1' => t('Yes, use queue module'), 
    '0' => t('No, do not use queue module'));
	} else {
		$options = array (
    '0' => t('No, the queue module is not enabled'));
	}

	$form['queue'] = array (
		'#type' => 'fieldset',
		'#title' => t('Automatic voting rules'), 
    '#collapsible' => TRUE, 
    '#collapsed' => $optins['1'] ? FALSE : TRUE 
    );

	$form['queue']['use_queue'] = array (
		'#type' => 'radios',
		'#title' => t('Queue and article publication'), 
    '#default_value' => isset($options['1']) ? $edit['use_queue'] : 0, 
    '#options' => $options, 
    '#description' => t('By default, choice, whether document is published or not, is completely in the hands of &quot;administrators&quot; (i.e. people with role &quot;administer nodes&quot;). But e-journal module can profit from the submission queues (if queue module is enabled) and may <b>automatically publish</b> unpublished nodes. The chief editor may publish individual nodes manually from the issue management page, simply by changing the combo box value, however, when submission queue is enabled, ejournal module may automatically publish all articles that got the <b>score above</b> the defined threshold. See option below.'), 
    '#required' => 1,
    );

	$post = drupal_map_assoc(array_reverse(array ( 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 20, 25, 30, 35, 40, 45, 50, 60, 70, 80, 90, 100, 10000)));
  
	$form['queue']['queue_threshold'] = array (
		'#type' => 'select',
		'#title' => t('Queue threshold'), 
    '#default_value' => $edit['queue_threshold'], 
    '#options' => $post, 
    '#description' => t('Queue module has its own threshold when documents get promoted, see !queue . But our threshold tells e-journal module when to <b>automatically publish</b> the unpublished node, that means, when the score <b>reaches</b> the limit, the status of the node will be changed from &quot;unpublished&quot; to &quot;published&quot;. If you only want to see votes/score but do not want to use automatic publishing, select the highest possible threshold.', array ('!queue' => l('queue module','admin/settings/queue'))),
    );


	// prepare nodetypes that are free
	foreach (explode(',', $edit['node_types']) as $type) {
		unset ($nodetypes[$type]);
	}
	// call plugins
	if ($plugins = ejournal_invoke_extensions('get_journal_form', $form, array('journal' => $journal, 'nodetypes' => $nodetypes, 'edit' => $edit))) {
		$form['plugins'] = array (
			'#type' => 'fieldset',
			'#title' => t('E-journal plugins'), 
      '#collapsible' => TRUE, 
      '#collapsed' => FALSE, 
      '#weight' => 9, 
      'form' => $plugins
      );
	} 
  
	$form['op'][] = array (
		'#type' => 'submit',
		'#value' => t('Submit'),
    '#weight' => 30,
    );

	if ($edit["jid"]) {
		$form['jid'] = array (
			'#type' => 'hidden',
			'#value' => $edit['jid'],
		  );
		$form['op'][] = array (
			'#type' => 'submit',
			'#value' => t('Delete'),
      '#weight' => 30,
      );
	}

	return $form;
}

/** 
 * ejournal_edit_journal: save changes to a journal
 *
 * @param $edit
 *   an array of values to be saved
 *
 * @return
 *   the same array, with modifications if a new record is created 
 */
function ejournal_journal_edit($edit) {

	if ($edit['jid']) {

		$ret = db_query("UPDATE {ejournal_journals} SET vid='%s', title = '%s', description = '%s', path='%s', about_page='%s', ejournal_layout='%s', archive_order='%s', enabled='%d', issn='%s', publisher_name ='%s', publisher_url='%s', publisher_email='%s', issue_format='%d' WHERE jid = '%d'", implode(',', (array) $edit['vid']), $edit['title'], $edit['description'], urlencode($edit['path']), $edit['about_page'], $edit['ejournal_layout'], $edit['archive_order'], $edit['enabled'], $edit['issn'], $edit['publisher_name'], $edit['publisher_url'], $edit['publisher_email'], $edit['issue_format'], $edit['jid']);

		if ($ret) {
			drupal_set_message(t('The journal %title has been updated.', array (
				'%title' => $edit['title']
			)));
		}
	} else {
    $edit['new_jid'] = TRUE;
		$edit['jid'] = db_next_id('{ejournal_journals}_jid');
		$ret = db_query("INSERT INTO {ejournal_journals} (jid, vid, title, description, path, about_page, ejournal_layout, archive_order, enabled, issn, publisher_name, publisher_url, publisher_email, issue_format) VALUES ('%d', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%s', '%s', '%d')", $edit['jid'], implode(',', (array) $edit['vid']), $edit['title'], $edit['description'], urlencode($edit['path']), $edit['about_page'], $edit['ejournal_layout'], $edit['archive_order'], $edit['enabled'], $edit['issn'], $edit['publisher_name'], $edit['publisher_url'], $edit['publisher_email'], $edit['issue_format']);

		if ($ret) {
			drupal_set_message(t('The journal %title has been created.', array (
				'%title' => $edit['title']
			)));
		}
	}
  ejournal_invoke_extensions('journal_edited', $edit);
	ejournal_journal_store_settings($edit);

	return $edit;
}

/** 
 * ejournal_journal_delete: delete a journal
 *
 * @param $edit
 *   associative array of journal parameters passed from HTML form
 *
 * @return
 *   A message stating that the publication has been deleted 
 */
function ejournal_journal_delete($edit) {
	if (!$edit['jid']) {
		return;
	}

	if (db_query("DELETE FROM {ejournal_journals} WHERE jid = '%d'", $edit['jid'])) {
		ejournal_journal_delete_settings($edit);
		drupal_set_message(t('The journal %title has been deleted.', array (
			'%title' => $edit['title']
		)));
	}

	if (db_query("DELETE FROM {ejournal_issues} WHERE jid = '%d'", $edit['jid'])) {
		drupal_set_message(t('All issues of the journal %title have been deleted.', array (
			'%title' => $edit['title']
		)));
	}

	if (db_query("DELETE FROM {ejournal_email_templates} WHERE jid = '%d'", $edit['jid'])) {
		drupal_set_message(t('All email templates of the journal %title have been deleted.', array (
			'%title' => $edit['title']
		)));
	}

	if ($edit['delete_everything']) {
		ejournal_delete_everything($edit['jid'], NULL);
	} else {
		$logdel = db_query("SELECT nid FROM {ejournal_articles} WHERE jid='%d'", $edit['jid']);
		while ($del = db_fetch_object($logdel)) {
			db_query("DELETE FROM {ejournal_articles} WHERE nid='%d'", $del->nid);
      ejournal_log_delete($del->nid, 'node');
		}
	}

	if ($edit['delete_everything']) {
		drupal_set_message(t('Attempts were made to delete all documents associated with the journal %title.', array (
			'%title' => $edit['title']
		)));
	} else {
		drupal_set_message(t('Documents associated with the journal %title were not deleted (only their links to the journal were removed).', array (
			'%title' => $edit['title']
		)));
	}

	ejournal_invoke_extensions('journal_deleted', $edit);
}

/** 
 * ejournal_journal_confirm_delete_form: confirm before deleting a publication
 *
 * @param $edit
 *   the publication ID to be deleted
 *
 * @return
 *   An array of form elements
 */
function ejournal_journal_confirm_delete_form($edit) {

	$form['confirm'] = array (
		'#type' => 'hidden',
		'#value' => 1
	);
	$form['jid'] = array (
		'#type' => 'hidden',
		'#value' => $edit['jid']
	);
	$form['title'] = array (
		'#type' => 'hidden',
		'#value' => $edit['title']
	);

	$form['delete_everything'] = _delete_everything_form('journal', $edit['delete_everything']);

	$form['submit'] = array (
		'#type' => 'item',
		'#value' => t('Delete journal "%title"', array ('%title' => $edit['title'])), 
    '#description' => t('Are you sure you want to delete the journal and all its issues?'));
	$form['op'] = array (
		'#type' => 'submit',
		'#value' => t('Delete'
	));

  ejournal_invoke_extensions('journal_delete_confirmation', $form, $edit);
	return $form;
}

function ejournal_journal_store_settings($edit = array ()) {
	if (!$edit['jid']) {
		return;
	}

	$keys = array (
		'year',
		'volume',
		'number',
		'issues_per_volume',
		'volumes_per_year',
		'node_types',
		'use_queue',
		'queue_threshold',
		'author_vid',
		'other_vid',
		'shortly_node_types'
	);

	if ($plugin_keys = ejournal_invoke_extensions('register_journal_value', $edit)) {
    foreach($plugin_keys as $value) {
        $keys[] = $value;
    }
	}

	foreach ($keys as $key) {
		if (isset ($edit[$key])) {
			db_query("DELETE FROM {ejournal_store} WHERE jid='%d' AND jkey='%s'", $edit['jid'], $key);
			if (is_array($edit[$key])) {
				foreach ($edit[$key] as $value) {
					db_query("INSERT INTO {ejournal_store} (jid, jkey, jvalue) VALUES ('%d', '%s', '%s')", $edit['jid'], $key, $value);
				}
			} else {
				db_query("INSERT INTO {ejournal_store} (jid, jkey, jvalue) VALUES ('%d', '%s', '%s')", $edit['jid'], $key, $edit[$key]);
			}
		}
	}

}

function ejournal_journal_delete_settings($edit = array ()) {
	if (!$edit['jid']) {
		return;
	}

	$keys = array (
		'year',
		'volume',
		'number',
		'issues_per_volume',
		'volumes_per_year',
		'node_types'
	);

	if ($plugin_keys = ejournal_invoke_extensions('delete_settings', $edit)) {
		$keys = array_merge($keys, $plugin_keys);
	}

	foreach ($keys as $key) {
		db_query("DELETE FROM {ejournal_store} WHERE jid='%d' AND jkey='%s'", $edit['jid'], $key);
	}

}

/**
 * ejournal_layouts: Loads and lists all available journal layouts.
 *
 * Layouts are loaded from files in the ejournal directory. Each layout file should have the filename
 *    layout_{name}.inc
 * It should also include a function with the filename and parameters
 *    theme_ejournal_layout_{name}($section, $params=NULL)
 *
 * @param $include_default
 *   Whether to return array with first option as <default> indicating that layout will be handled by functions from ejournal.module
 *
 * @return
 *   An array of layouts and their descriptions, suitable for use as the $options in a form_select call, e.g.,
 *      form_select($title, $name, $value, ejournal_layouts(), $description)
 */
 
function ejournal_layouts($exclude_default = TRUE) {

  $layouts = array ();
  $layouts[0] = '<' . t('none'). '>';
  $files = file_scan_directory(drupal_get_path('module', 'ejournal'), '^theme_.*\.inc$');
  foreach ($files as $filename => $file) {
    $name = substr($file->name, 6);
    $layouts[$name] = $name;
  }
  if($exclude_default) {
    unset($layouts['standard']);
  }
  return $layouts;
}


?>
