<?php



/*******************************************
 *   VOLUMES
 ******************************************/
/** 
 * ejournal_volume_edit: return a page for editing a journal volume
 *
 * @param $edit
 *   an array of values handled by the form
 *
 * @return
 *   An HTML-formatted form 
 */
function ejournal_volume_edit($edit = array ()) {
	$journal = ejournal_journal_get($edit['jid']);

	$output = drupal_get_form('ejournal_volume_edit_form', $edit);

	$output .= "<hr/>\n";

	$issue_statuses = _ejournal_issue_statuses();

	$issues = _ejournal_journal_get_issues($journal, $edit['volume'], TRUE);

	if (!$issues) {
		$output .= '<p>' . t('There were found no issues with this identification - volume: "%volume".', array (
			'%volume' => $edit['volume']
		)) . "</p>\n";
	} else {
		$header = array (
			t('Issue') . 'ID', 
      t('Year'), 
      t('Volume'), 
      t('Number'), 
      t('Custom identification'), 
      t('Published'), 
      t('Status'), 
      t('Description')
      );

		$output .= '<p>' . t('All the issues below will be affected.') . "</p>\n";

		while ($issue = db_fetch_object($issues)) {
			$rows[] = array (
				l($issue->iid, ejournal_admin_path_set(array('action' => 'edit', 'iid' => $issue->iid))), $issue->year, 
        $issue->volume, 
        $issue->number, 
        $issue->custom_identification, 
        format_date($issue->date_published), 
        $issue_statuses[$issue->status], 
        $issue->description,
        );
		}
		$output .= theme('table', $header, $rows);
	}
	return $output;
}

function ejournal_volume_edit_form($edit) {
	$journal = ejournal_journal_get($edit['jid']);

	$form['x1'] = array (
		'#type' => 'item',
		'#value' => '<h3>' . t('Volume %vol of journal "!journal"',
		array (
			'%vol' => $edit['volume'],
			'!journal' => l($journal->title,
			"admin/settings/ejournal/edit/$journal->jid"
		)
	)) . "</h3>\n");

	$form['jid'] = array (
		'#type' => 'hidden',
		'#value' => $edit['jid']
	);
	$form['volume_old'] = array (
		'#type' => 'hidden',
		'#value' => $edit['volume']
	);
	$form['year_old'] = array (
		'#type' => 'hidden',
		'#value' => $edit['year']
	);

	$form['year'] = array (
		'#type' => 'textfield',
		'#title' => t('Year'
	), '#default_value' => $edit['year'], '#size' => 50, '#maxlength' => 64, '#description' => t('If your journal use identification by year of publication, you may change all the values in the year fields now.'),);

	$form['volume'] = array (
		'#type' => 'textfield',
		'#title' => t('Volume'
	), '#default_value' => $edit['volume'], '#size' => 50, '#maxlength' => 64, '#description' => t('A number (or name) for this volume. Since many publications begin a new volume annually, the name might simply be the name of that year, e.g., "2004", or digit, e.g., "2"'),);

	if ($plugins = ejournal_invoke_extensions('volume_edit_form', $form, array('edit' => $edit, 'journal' => $journal))) {
		$form['plugins'] = array (
			'#type' => 'fieldset',
			'#title' => t('E-journal plugins'), 
      '#weight' => 30, 
      'form' => $plugins
      );
	}

	$form['op'] = array (
		'#type' => 'submit',
		'#value' => t("Submit")
    );
	$form['delete'] = array (
		'#type' => 'submit',
		'#value' => t("Delete")
    );
  
  
	return $form;
}

/** 
 * ejournal_volume_process: save changes to a journal volume {ejournal_issues} table
 * makes only updates and also only if the issue(s) already exists
 *
 * @param $edit
 *   an array of values to be saved
 *
 * @return
 *   the same array 
 */
function ejournal_volume_process($edit) {
	$journal = ejournal_journal_get($edit['jid']);
	$issues = _ejournal_journal_get_issues($journal, $edit['volume_old'], TRUE);

	if ($issues) {
		// _old = current value in the table, and we want to replace it with new value
		if ($edit['volume'] != $edit['volume_old'] || ($edit['year'] != $edit['year_old'])) {
			$cond = "volume='" . db_escape_string($edit['volume_old']) . "' ";
			$set = "volume='" . db_escape_string($edit['volume']) . "' ";
		}
		if ($edit['year'] != $edit['year_old']) { // user wanted to update year in all issues of the volume, year_old is always empty value when editing volume with >1 issues
			$set .= $set ? ", " : '';
			$set .= "year='" . db_escape_string($edit['year']) . "' ";
		}

		if ($set && $cond) {
			while ($issue = db_fetch_object($issues)) {
				db_query("UPDATE {ejournal_issues} SET $set WHERE $cond");
			}
		}
	} else {
		drupal_set_message(t('There are no issues in volume %volume of journal %journal. Nothing was updated', array (
			'%volume' => $edit['volume_old'],
			'%journal' => $journal->title
		)));
	}

	ejournal_invoke_extensions('volume_edited', $edit, array('journal' => $journal, 'issues' => $issues));

	return $edit;
}
/** 
 * ejournal_volume_delete: delete a journal volume (all its issues)
 *
 * @param $edit
 *   an array from the HTML form specifying the volume to be deleted
 *
 * @return
 *   A message stating that the volume has been deleted 
 */
function ejournal_volume_delete($edit = array ()) {
	$journal = ejournal_journal_get($edit['jid']);

	if ($edit['delete_everything']) {
		$type = t('articles');
		if ($issues = _ejournal_journal_get_issues($journal, $edit['volume'], TRUE)) {
			while ($issue = db_fetch_object($issues)) {
				$affected += ejournal_delete_everything($edit['jid'], $issue->iid);
			}
		} else {
			$affected = 0;
		}
		db_query("DELETE FROM {ejournal_issues} WHERE jid = '%d' AND volume = '%s'", $edit['jid'], $edit['volume']);
	} else {
		$result = db_query("DELETE FROM {ejournal_issues} WHERE jid = '%d' AND volume = '%s'", $edit['jid'], $edit['volume']);
		$affected = db_affected_rows($result);
		$type = t('issues');

	}

	if ($affected) {
		drupal_set_message(t('Deleted %num %type identified by volume "%volume" from journal "%journal".', array (
			'%num' => $affected,
			'%type' => $type,
			'%volume' => $edit['volume'],
			'%journal' => $journal->title
		)));
	} else {
		drupal_set_message(t('There are no %type in volume %volume of journal %journal. Nothing was deleted (but the empty volume)', array (
			'%volume' => $edit['volume'],
			'%type' => $type,
			'%journal' => $journal->title
		)));
	}

	ejournal_invoke_extensions('volume_deleted', $edit, array('journal' => $journal, 'issues' => $issues));
}

/** 
 * ejournal_volume_confirm_delete_form: confirm before deleting a journal volume
 *
 * @param $edit
 *   an array with the values specifying the volume to be deleted
 *
 * @return
 *   An array of form elements
 */
function ejournal_volume_confirm_delete_form($edit) {
	$journal = ejournal_journal_get($edit['jid']);

	$form['confirm'] = array (
		'#type' => 'hidden',
		'#value' => 1
	);
	$form['jid'] = array (
		'#type' => 'hidden',
		'#value' => $edit['jid']
	);
	$form['volume'] = array (
		'#type' => 'hidden',
		'#value' => $edit['volume']
	);

	$form['op'] = array (
		'#type' => 'hidden',
		'#value' => t('Delete')
    );

	$form['desc'] = array (
		'#type' => 'item',
		'#value' => t('Do you really want to delete volume "%volume" (all its issues!)  from the journal "%journal"?',		array (
			'%journal' => $journal->title,
			'%volume' => $edit['volume'],			
      )), 
    '#description' => t('Are you sure you want to delete this volume name? If you do, the volume number will not be displayed in its place.')
    );
  
    ejournal_invoke_all('volume_delete_confirmation', $form, $edit);
    
	return $form;
}

/*******************************************
 *   ISSUES
 ******************************************/
/** 
 * ejournal_issue_edit_form: return a form for editing a journal issue
 *
 * @param $edit
 *   an array of values handled by the form
 *
 * @return
 *   An array of form elements
 */
function ejournal_issue_edit_form($edit = array ()) {

	$journal = ejournal_journal_get($edit['jid']);
	if ($edit['iid']) {
		$edit = (array) (ejournal_issue_get($edit['iid']));
	}

	$form['issue'] = array (
		'#type' => 'fieldset',
		'#title' => '<h3>' . t('Issue of journal ') . l($journal->title, "admin/settings/ejournal/edit/$journal->jid") . "</h3>\n", 
    '#prefix' => '<div class="container-inline">', 
    '#suffix' => '</div>',);

	if ($edit['iid']) {
		$form['issue']['#description'] = "<p>" . t('You may also !move articles between issues', array('!move' => l('move', ejournal_admin_path_set(array('action' => 'manage', 'jid' => $journal->jid, 'iid' => $edit['iid']))))) . "</p>\n";
	}

	$form['issue']['jid'] = array (
		'#type' => 'hidden',
		'#value' => $journal->jid
	);

	if ($journal->issue_format == 4) {
		$form['issue']['custom_identification'] = array (
			'#type' => 'textfield',
			'#title' => t('Custom identification'), 
      '#default_value' => $edit['custom_identification'], '#size' => 50, '#maxlength' => 128, '#description' => t('Your own identification of the issue, e.g., "March 2003" or "April 11-17, 2004."'),);
	} else {
		$last_hint = _ejournal_issue_get_increment($journal);
		if ($journal->issue_format > 0) {
			$form['issue']['year'] = array (
				'#prefix' => '<b>' . t('Year') . ':</b>', 
        '#type' => 'textfield', 
        '#default_value' => $edit['iid'] ? $edit['year'] : $last_hint['year'], 
        '#size' => 6, 
        '#maxlength' => 6,
        );
		}
		if ($journal->issue_format > 1) {
			$form['issue']['volume'] = array (
				'#type' => 'textfield',
				'#default_value' => $edit['iid'] ? $edit['volume'] : $last_hint['volume'],
				'#size' => 6,
				'#maxlength' => 6,
				'#prefix' => '<b>' . t('Volume') . ':</b>',
        );
		}
		if ($journal->issue_format > 2) {
			$form['issue']['number'] = array (
				'#type' => 'textfield',
				'#default_value' => $edit['iid'] ? $edit['number'] : $last_hint['number'],
				'#size' => 6,
				'#maxlength' => 6,
				'#prefix' => '<b>' . t('Number'
			) . ':</b>',);
		}
		// issue date
		$form['issue']['issue_publication_time'] = _ejournal_date_form($edit['date_published'], 'issue');

		$form['issue']['x1'] = array (
			'#type' => 'item',
			'#value' => "<br/><br/>(" . t('The last issue : <b>%date</b>, Year <b>%year</b>, Vol. <b>%volume</b>, No. <b>%number</b>',
			array (
				'%date' => $last_hint['issue_last_time'] ? format_date($last_hint['issue_last_time']
			) : t('none'), '%year' => $last_hint['issue_last_year'], '%volume' => $last_hint['issue_last_volume'], '%number' => $last_hint['issue_last_number'])) . ')');

		if ($journal->issue_format == 3 && !$edit['date_published']) {
			$edit['date_published'] = $last_hint['time'];
		}
	}

	$form['description'] = array (
		'#type' => 'textarea',
		'#title' => t('Description'
	), '#default_value' => $edit['description'], '#size' => 70, '#maxlength' => 4,);

	// Issue status
	$issue_status = _ejournal_issue_statuses();
	$form['status'] = array (
		'#type' => 'select',
		'#title' => t('Issue status'), 
    '#default_value' => $edit['status'], 
    '#options' => $issue_status, 
    '#description' => t('Choose status of this issue. <b>Current issue</b> is the one that is on the front page. <b>Back issues</b> are the published issues (i.e. non-current). <b>Future issue</b> will be opened (automatically) at the specified date/hour in the future (becoming current issue). <b>Not published</b> issues remain invisible (only chief editor can see them), <b>working issues</b> are invisible to readers, but editors and authors can send articles to them.'),);

	$form['cover_page_nid'] = array (
		'#type' => 'textfield',
		'#title' => t('Cover page'), 
    '#default_value' => $edit['cover_page_nid'], 
    '#size' => 6, 
    '#maxlength' => 6, 
    '#description' => t('Number of the node which should be displayed as a cover page of this issue'),);

	$form['op'] = array (
		'#type' => 'submit',
		'#value' => t("Submit"
	));

	//building form for management of articles which were attached to this issue
	if (isset ($edit["iid"])) {
		$form['delete']['op'] = array (
			'#type' => 'submit',
			'#value' => t("Delete")
      );
		$form['iid'] = array (
			'#type' => 'hidden',
			'#value' => $edit['iid']
		);

		$form['articles'] = array (
			'#type' => 'fieldset',
			'#title' => t('Manage articles'),
      '#description' => t('Please note, the selection form for taxonomy terms is simplified. If more than two vocabularies are assigned to a document, then you should see multiple selection form with both (all) terms of these vocabularies. You should not select multiple terms for the vocabulary that does not allow !multiple choices.'),
      '#expandable' => TRUE, 
      '#expanded' => TRUE,
      );

		// Select all articles associated with this issue
		$issue = ejournal_issue_get($edit['iid']);
		$queue = $journal->use_queue;
		$operations = ejournal_article_handle_operations('get');

		$form['articles']['operation'] = array (
			'#type' => 'fieldset',
			'#prefix' => '<div class="container-inline">',
			'#suffix' => '</div>'
		);
		$form['articles']['operation']['operation'] = array (
			'#type' => 'select',
			'#options' => $operations
		);

		$header = array (
			NULL,
			t('Article') . "/" . t('Type'), 
      t('Published'), 
      ($queue ? t('Moderated') : ''), 
      ($queue ? t('Score') : ''), 
      t('Approved'), 
      t('Weight'), 
      t('Section')
      );
		$yes_no = array (
      0 => t('no'), 
      1 => t('yes'),      
      );
    $proof_yes_no = array (
      PROOFREADER_NOT_APPROVED => t('no'), 
      PROOFREADER_APPROVED => t('yes'),
      PROOFREADER_IGNORED => '---',
      );

		$articles = _ejournal_issue_get_articles($journal, $edit['iid'], TRUE);

		foreach ($articles as $node) {

      $terms = ejournal_sections_get($journal->jid, $node->type);
      
			$rows[] = array (
				theme('hidden', array (
					'#name' => 'edit[nid][' . $node->nid . ']',
					'#id' => 'edit-nid][' . $node->nid,
					'#value' => $node->nid
				)) .
        theme('checkbox', array (
          '#name' => 'edit[nid_checked][' . $node->nid . ']',
          '#id' => 'edit-nid_checked][' . $node->nid,
          '#return_value' => 1,
          '#value' => 0,
          '#parents' => array (
            'nid_checked][' . $node->nid
          )
        )), 
        l($node->title, 'node/' . $node->nid . '/edit?destination=' . urlencode(ejournal_admin_path_set(array('action' => 'edit', 'jid' => $journal->jid, 'iid' => $edit['iid'])))) . ' ' . theme('mark', node_mark($node->nid, $node->changed)) . ' / ' . node_get_types('name', $node), 
        theme('select', array (
          '#name' => 'edit[node_status][' . $node->nid . ']',
          '#id' => 'edit-node_status][' . $node->nid,
          '#options' => $yes_no,
          '#value' => $node->status,
          '#parents' => array (
            'node_status][' . $node->nid
          ))), 
        ($queue ? $yes_no[$node->moderate] : ''), 
        ($queue ? _ejournal_article_queue_status($node->nid) : ''), 
        t($proof_yes_no[$node->p_status]),
        them_ejournal_weight( array (        
          '#name' => 'edit[node_weight][' . $node->nid . ']',
          '#id' => 'edit-node_weight][' . $node->nid,
          '#delta' => 10,
          '#value' => $node->article_weight,
          //'#options' => process_weight(array('#delta'=> 10)),
          //'#options' => array( '#process' => array('process_weight' => array('#delta'=> 10))),
          '#parents' => array (
            'node_weight][' . $node->nid
            )
          )
        ), 
        theme('select', array (
          '#name' => 'edit[section_tid][' . $node->nid . ']',
          '#id' => 'edit-section_tid][' . $node->nid,
          '#options' => array_merge(array('0' => '<' . t('none'). '>'), $terms),
          '#value' => $node->tid,
          '#multiple' => count($terms) > 1 ? TRUE : FALSE, 
          '#parents' => array ('section_tid][' . $node->nid)
          )
        ) . theme('hidden', array (
              '#name' => 'edit[old_tid][' . $node->nid . ']',
              '#id' => 'edit-old_tid][' . $node->nid,
              '#value' => implode(",",	array_keys(ejournal_sections_of_article($journal->jid, $node->type, $node->tid)))
              )
            ),
      );
		}
    
		$group = theme('table', $header, $rows);

		$form['articles']['manage'] = array (
			'#type' => 'item',
			'#value' => $group,
			
		);
		drupal_set_title(t('edit ') . "$journal->title, " . ejournal_issue_reference($issue, $journal));
	}

	if ($plugins = ejournal_invoke_extensions('issue_edit_form', $form, array('edit' => $edit, 'issue' => $issue, 'journal' => $journal))) {
		$form['plugins'] = array (
			'#type' => 'fieldset',
			'#title' => t('E-journal plugins'), 
      '#weight' => 30, 
      'form' => $plugins
      );
	}

	return $form;
}

/**
 *  ejournal_issue_manage : renders the main page with available articles for the
 *  issue in question (only journals available to the current user
 *  are shown)
 *  This page is used for moving articles from one issue to another
 *  or for WORKING area (moving articles that don't belong to any issue)
 * 
 *  @param $jid
 *    comma delimited id(s) of journals (if empty, all journals are listed)
 *  @param $iid
 *    constant, type of the issue to render
 *  @return
 *    An HTML-rendered section with form(s)
 */

function ejournal_issue_manage($jid = NULL, $iid = ISSUE_WORKING) {

	if ($jid) {
		$jids = split(',', $jid);
	} else {
    $jids = ejournal_journals_get_all();
	}

	foreach ((array)$jids as $jid) {
		$journal = ejournal_journal_get($jid);
		$journal->iid = $iid; //normally, journal->iid is not set
		$output .= "<h3>" . t($journal->title) . "</h3>\n";
		$output .= drupal_get_form('ejournal_issue_manage_form' . $journal->jid, $journal);
	}

	return $output;
}
/**
 * ejournal_issue_manage_form: returns the same for different ejournals, 
 * it is a dynamic form of construction of forms, usually called via hook ejournal_forms 
 */
function ejournal_issue_manage_form() {
	$journal = func_get_arg(0);
	
	$order = variable_get('ejournal_issue_order', 'DESC');
	$issue_status = _ejournal_issue_statuses();
	$yes_no = array (
		'0' => t('no'), 
		'1' => t('yes'),
		);
  $proof_yes_no = array (
		PROOFREADER_NOT_APPROVED => t('no'), 
		PROOFREADER_APPROVED => t('yes'),
    PROOFREADER_IGNORED => '---',
		); 
	$operations = ejournal_article_handle_operations('get');
	
	$form = array ();
	$form['articles'] = array (
		'#type' => 'fieldset',
		'#title' => t('Manage articles'), 
    '#expandable' => TRUE, 
    '#expanded' => TRUE,
    );

	$form['articles']['jid'] = array (
		'#type' => 'hidden',
		'#value' => $journal->jid
	);

	$form['articles']['operation'] = array (
		'#type' => 'fieldset',
		'#prefix' => '<div class="container-inline">',
		'#suffix' => '</div>'
	);
	$form['articles']['operation']['operation'] = array (
		'#type' => 'select',
		'#defalut_value' => '',
		'#options' => $operations
	);
	$form['articles']['operation']['op'] = array (
		'#type' => 'submit',
		'#value' => t("Submit")
  );
	

	if (!ejournal_user_access(array ('chief editor'), $journal)) {
		next;
	}

  $admin_path = ejournal_admin_path_get();
  
	$destination = ($admin_path['action'] == 'working') ? 
                 urlencode(ejournal_admin_path_set(array('action' => 'working'))) :
                 urlencode(ejournal_admin_path_set(array('action' => 'manage', 'jid' => $journal->jid)));  //normally, journal->iid is not set

	$issues = ejournal_issues_get_all_available($journal->jid);
	$queue = $journal->use_queue;
	$header = array (
		NULL,
		t('Article') . '/' . t('Type'), 
    t('Published'), 
    ($queue ? t('Moderated') : ''), 
    ($queue ? t('Score') : ''), 
    t('Approved'), 
    t('Available issues'), 
    t('Section')
  );

	$result = _ejournal_issue_get_articles($journal, $journal->iid, TRUE);

	foreach ($result as $node) {
		$rows[] = array (
			theme('hidden',	array (
				'#name' => 'edit[nid][' . $node->nid . ']',
				'#id' => 'edit-nid][' . $node->nid,
				'#value' => $node->nid
			  )
      ) .	theme('checkbox', array (
            '#name' => 'edit[nid_checked][' . $node->nid . ']',
            '#id' => 'edit-nid_checked][' . $node->nid,
            '#return_value' => 1,
            '#value' => 0,
            '#parents' => array (
              'nid_checked][' . $node->nid
              )
            )
          ),
      l($node->title, 'node/' . $node->nid . '/edit' . '/edit?destination=' . $destination) . ' ' . theme('mark', node_mark($node->nid, $node->changed)) . ' (' . node_get_types('name', $node) . ')', 
      theme('select', array (
        '#name' => 'edit[node_status][' . $node->nid . ']',
        '#id' => 'edit-node_status][' . $node->nid,
        '#options' => $yes_no,
        '#value' => $node->status,
        '#parents' => array (
          'node_status][' . $node->nid
          )
        )
      ), 
      ($queue ? $yes_no[$node->moderate] : ''), 
      ($queue ? _ejournal_article_queue_status($node->nid) : ''), 
      t($proof_yes_no[$node->p_status]), 
      theme('select', array (
        '#name' => 'edit[nid_issue][' . $node->nid . ']',
        '#id' => 'edit-nid_issue][' . $node->nid,
        '#options' => $issues,
        '#value' => $node->iid,
        '#parents' => array (
          'nid_issue][' . $node->nid
          )
        )
      ), 
      theme('select', array (
        '#name' => 'edit[section_tid][' . $node->nid . ']',
        '#id' => 'edit-section_tid][' . $node->nid,
        '#options' => ejournal_sections_get($journal->jid, $node->type), 
        '#value' => $node->tid, 
        '#parents' => array (
          'section_tid][' . $node->nid
          )
        )
      ) .		theme('hidden', array (
              '#name' => 'edit[old_tid][' . $node->nid . ']',
              '#id' => 'edit-old_tid][' . $node->nid,
              '#value' => implode(",", array_keys(ejournal_sections_of_article($journal->jid, $node->type, $node->tid)))
              )
            ),
    );
	}

	$group = theme("table", $header, $rows);

	$form['articles']['manage'] = array (
		'#type' => 'item',
		'#value' => $group,
	);

	return $form;
}

/** 
 * ejournal_issue_edit: save changes to a journal issue
 *
 * @param $edit
 *   an array of values to be saved
 * @param $type
 *   not used for now
 */
function ejournal_issue_edit($edit, $type = 'issue') {
	if (!$edit['jid']) {
		return;
	};

	$timestamp = _ejournal_validate_date($edit, $type);
	$edit['date_published'] = $timestamp;

	// old current_issue becomes back issue
	if ($edit['status'] == ISSUE_CURRENT) {
		db_query("UPDATE {ejournal_issues} SET status='%d' WHERE status='%d' AND jid='%d'", ISSUE_PUBLISHED, ISSUE_CURRENT, $edit['jid']);
	}

	if ($edit['iid']) {

		db_query("UPDATE {ejournal_issues} SET jid='%d', year='%s', volume='%s', number='%s', custom_identification='%s', cover_page_nid='%s', date_published='%d', status='%d', issue_order='%d', description='%s' WHERE iid = '%d'", $edit['jid'], $edit['year'], $edit['volume'], $edit['number'], $edit['custom_identification'], $edit['cover_page_nid'], $timestamp, $edit['status'], $edit['issue_order'], $edit['description'], $edit['iid']);

		// Handle operations: 
		if ($edit['operation']) {
			ejournal_article_handle_operations($edit);
			drupal_set_message(t('The update has been performed.'));
		}
	} else {
		$edit['iid'] = db_next_id('{ejournal_issues}_iid');
		db_query("INSERT INTO {ejournal_issues} (iid, jid, year, volume, number, custom_identification, cover_page_nid, date_published, status, issue_order, description) VALUES ('%d', '%d', '%s', '%s', '%s', '%s', '%s', '%d', '%d', '%d', '%s')", $edit['iid'], $edit['jid'], $edit['year'], $edit['volume'], $edit['number'], $edit['custom_identification'], $edit['cover_page_nid'], $timestamp, $edit['status'], $edit['issue_order'], $edit['description']);
	}

	// now grant access to all articles that are in this issue 
	if ($edit['status'] == ISSUE_CURRENT) {
    
		ejournal_invoke_extensions('issue_published', $edit);      
      
		$current_issue = (array)ejournal_global_variable('ejrn_current_issue');
		$current_issue[$edit['jid']] = (array) (array (
			'iid' => $edit['iid'],
			'jid' => $edit['jid'],
			'year' => $edit['jid'],
			'volume' => $edit['volume'],
			'number' => $edit['number'],
			'custom_identification' => $edit['custom_identification'],
			'date_published' => $timestamp,
			'description' => $edit['description']
		)
    );

		//store the issue identification, for the template to use
		ejournal_global_variable('ejrn_current_issue', $current_issue);
		_ejournal_set_ejrn_next_issue($edit);
	} 
  else if ($edit['status'] == ISSUE_FUTURE) {
			_ejournal_set_ejrn_next_issue($edit);
	}

	ejournal_invoke_extensions('issue_edited', $edit);
}

function ejournal_issue_manage_edit($edit) {
	// Handle operations:        
	if ($edit['operation']) {
		ejournal_article_handle_operations($edit);
		drupal_set_message(t('The update has been performed.'));
	}

  ejournal_invoke_extensions('issue_edited', $edit);

	return $edit;
}

/** 
 * ejournal_issue_delete: delete a journal issue
 *
 * @param $edit
 *   an array for the issue to be deleted
 *
 * @return
 *   nothing 
 */
function ejournal_issue_delete($edit = array ()) {

	$journal = ejournal_journal_get($edit['jid']);
	$issue = ejournal_issue_get($edit['iid']);

	if ($edit['delete_everything']) {
		ejournal_delete_everything($edit['jid'], $edit['iid']);
	} else {
    $articles = db_query("SELECT nid FROM {ejournal_articles} WHERE iid='%d' and jid='%d'", $edit['iid'], $edit['jid']);
    while($n = db_fetch_object($articles)) {
      ejournal_log_delete($n->nid, 'node');
    }
		db_query("DELETE FROM {ejournal_articles} WHERE iid = '%d' AND jid='%d'", $edit['iid'], $edit['jid']);
	}

	db_query("DELETE FROM {ejournal_issues} WHERE iid = '%d' AND jid='%d'", $edit['iid'], $edit['jid']);

	drupal_set_message(t('Deleted issue "%issue" from journal "%journal."', array (
		'%issue' => ejournal_issue_reference($issue, $journal), '%journal' => $journal->title)));

  ejournal_log_delete($edit['iid'], 'issue');
	ejournal_invoke_extensions('issue_deleted', $edit, $journal, $issue);
}

/** 
 * ejournal_issue_confirm_delete_form: confirm before deleting a journal issue
 *
 * @param $edit
 *   an array for the issue to be deleted
 *
 * @return
 *   An array of form elements
 */

function ejournal_issue_confirm_delete_form($edit) {
	$journal = ejournal_journal_get($edit['jid']);

	$form['jid'] = array (
		'#type' => 'hidden',
		'#value' => $edit['jid']
	);
	$form['iid'] = array (
		'#type' => 'hidden',
		'#value' => $edit['iid']
	);
	$form['confirm'] = array (
		'#type' => 'hidden',
		'#value' => 1
	);

	$form['delete_all'] = _delete_everything_form('issue', $edit['delete_everything']);

	$form['item'] = array (
		'#type' => 'item',
		'#value' => t('Delete issue "%issue" from journal "%journal"?',
		array (
			'%issue' => ejournal_issue_reference((object) ($edit
		)
	), '%journal' => $journal->title)), '#description' => t('Are you sure you want to delete this issue?'),);
	$form['op'] = array (
		'#type' => 'submit',
		'#value' => t('Delete'
	));
	$form['cancel']['op'] = array (
		'#type' => 'submit',
		'#value' => t('Cancel'
	));
	return $form;
}
//should find the latest year|volume|number and return it plus 1
function _ejournal_issue_get_increment($journal, $this_year = NULL) {

	if ($journal->issue_format == 4) {
		return array ();
	}

	if (!$this_year) {
		$y = db_result(db_query("SELECT MAX(date_published) FROM {ejournal_issues} WHERE jid=%d", $journal->jid));
		$this_year = $y ? date('Y', $y) : date('Y', time());
	}

	$begin = mktime(0, 0, 0, 1, 1, $this_year);
	$end = mktime(0, 0, 0, 1, 1, $this_year +1);

	$issues = db_query("SELECT * FROM {ejournal_issues} WHERE date_published > %d AND date_published < %d AND jid=%d ORDER BY date_published ASC", $begin, $end, $journal->jid);

	$issue_count = 0;
	$volume_count = 0;

	$prev_vol = "";
	while ($issue = db_fetch_object($issues)) {
		if ($prev_vol != trim($issue->volume)) {
			$volume_count++; //number of volumes in this year
			$prev_vol = trim($issue->volume); // last volume
		}

		if ($issue_count) {
			$total_time += ($issue->date_published - $last->date_published);
		}
		$issue_count++;
		$last = $issue;
		$iss[$volume_count] = $iss[$volume_count] + 1; //number of issues in each volume of the year
	}

	$iss_per_vol = is_numeric($journal->issues_per_volume) ? $journal->issues_per_volume : FALSE;
	$vol_per_year = is_numeric($journal->volumes_per_year) ? $journal->volumes_per_year : FALSE;
	$prev_vol = empty ($prev_vol) ? 0 : $prev_vol;
	$prev_num = is_numeric($last->number) ? $last->number : FALSE;

	//full format year, volume, number  
	if ($journal->issue_format == 3) {
		$iss_per_vol = $iss_per_vol ? $iss_per_vol : 12;
		$vol_per_year = $vol_per_year ? $vol_per_year : 1;

		if ($issue_count >= ($iss_per_vol * $vol_per_year)) {
			$ret = array (
				'year' => $this_year +1,
				'last_year' => $this_year,
				'volume' => is_numeric($prev_vol
			) ? $prev_vol +1 : $prev_vol, 'last_volume' => $prev_vol, 'number' => 1, 'num_in_vol' => $issue_count,);
		} else {
			if ($volume_count < $vol_per_year) {
				if ($iss[$volume_count] < $iss_per_vol) {
					$volume = $prev_vol;
					$number = $iss[$volume_count] + 1;
				} else {
					$volume = $prev_vol +1;
					$number = 1;
				}
			} else {
				if ($iss[$volume_count] < $iss_per_vol) {
					$volume = $prev_vol;
					$number = $iss[$volume_count] + 1;
				} else {
					$volume = $volume_count +1;
					$number = 1;
				}
			}
			$ret = array (
				'year' => $this_year,
				'last_year' => $this_year -1,
				'volume' => $volume,
				'last_volume' => $prev_vol ? $prev_vol -1 : $prev_vol,
				'number' => $number,
				'last_number' => $last->number,
				'num_in_vol' => $issue_count
			);

		}

	}

	if ($issue_count > 1) {
		$ret['time'] = $last->date_published + ($total_time / $issue_count);
	} else {
		$ret['time'] = $last->date_published + (30 * 86400);
	}
	$ret['issue_last_year'] = $last->year;
	$ret['issue_last_volume'] = $last->volume;
	$ret['issue_last_number'] = $last->number;
	$ret['issue_last_time'] = $last->date_published;
	return $ret;

}

/*******************************************
 *                ARTICLES
 ******************************************/
function ejournal_article_handle_operations($edit) {
	if ($edit == 'get') {
		return array (
			'update_all' => t('Update all'), 
      'update_sel' => t('Update selected'), 
      'unpublish_all' => t('Send all to unpublished issue'), 
      'unpublish_sel' => t('Send selected to unpublished issue'), 
      0 => t('Do nothing'));
	}
  
  
  if(!$edit['edit']['nid']) {
    return;
  }
  
	foreach ($edit['edit']['nid'] as $nid) {    
		$checked = $edit['edit']['nid_checked'][$nid];
		$arg = array ();
		switch ($edit['operation']) {
			case 'update_sel' :
				if (!$checked) {
					break;
				}
			case 'update_all' :

				if ($edit['edit']['nid_issue'][$nid]) {
					$arg['iid'] = $edit['edit']['nid_issue'][$nid];
				}
				if ($edit['edit']['node_weight'][$nid]) {
					$arg['weight'] = $edit['edit']['node_weight'][$nid];
				}
				_ejournal_article_update($nid, $arg);
				ejournal_article_update_external($nid, array (
					'status' => $edit['edit']['node_status'][$nid],
					'tid' => (array)$edit['edit']['section_tid'][$nid],
					'old_tid' => $edit['edit']['old_tid'][$nid],
				));
				break;

			case 'unpublish_sel' :
				if (!$checked) {
					break;
				}
			case 'unpublish_all' :
				_ejournal_article_update($nid, array (
					'iid' => ISSUE_NOT_PUBLISHED
				));
				break;
		}
	}
}

/**
 * ejournal_article_insert: called from the nodeapi when article is inserted for the first time
 *
 * @param $jid
 *    Id of the journal
 * @param $node
 *    Object representing node
 */

function ejournal_article_insert($jid, &$node) {  
  // insert article
  ejournal_article_add($jid, $node);
  // logs, issues
  if($node->article_log) {
    ejournal_log_write($node->nid, 'node', 'message', $node->article_log);
  }
  ejournal_log_write($node->nid, 'node', 'notice', "Node has been created by: [user-name]");
  // send alert emails
  ejournal_email_article_inserted($jid, $node);
  
  ejournal_invoke_extensions('article_insert', $node, array('jid' => $jid));
  
}

/**
 * ejournal_article_update: called from the nodeapi when article is updated
 *
 * @param $jid
 *    Id of the journal
 * @param $node
 *    Object representing node
 */
function ejournal_article_update($jid, &$node) {
  $journal = ejournal_journal_get($jid);
  $article = ejournal_article_get($node->nid, $journal);
  // issue changed
  if ($article->iid != $node->nid_issue) {
    ejournal_log_write($node->nid, 'node', 'notice', t("issue changed from: %old to: %new", array ('%old' => $article->iid ? $article->iid : '-', '%new' => $node->nid_issue)));
    $update++;
  }
  // section changed, it will be handled by taxonomy module, we only log the operation
  if ($node->taxonomy && !in_array($article->tid, $node->taxonomy)) {
    ejournal_log_write($node->nid, 'node', 'notice', t("section has changed (old: %old)", array ('%old' => $article->tid ? $article->tid : '-' )));
    $update++;
  }
  // proofreader mark changed
  if ($node->proofreader_mark && $article->proofreader_mark != $node->proofreader_mark) {
    $proof = _ejournal_article_get_proofreader_statuses();
    ejournal_log_write($node->nid, 'node', 'notice', t("proofreader mark: " . $proof[$node->proofreader_mark]));
    $update++;
  }
  
  if($node->article_log) {
    ejournal_log_write($node->nid, 'node', 'message', $node->article_log);
  }

  if ($update) {
      ejournal_article_add($jid, $node);
  }
  
  ejournal_invoke_extensions('article_update', $node, array('journal' => $journal, 'article' => $article));
}

/**
 *  Updates only the specified columns, returns number of updated rows
 *  it is called for updates inside of issues, volumes (ie. not when editing article)
 */

function _ejournal_article_update($nid, $arg = array ()) {

	foreach ($arg as $key => $value) {
		if (isset ($value)) {
			$sql[] = "$key='" . db_escape_string($value) . "' ";
		}
	}
	if (!$sql) {
		return;
	}
	$set = implode(', ', $sql);

	$result = db_query("UPDATE {ejournal_articles} SET $set WHERE nid='%d'", $nid);
  return db_affected_rows($result);
}


/********************************************
 *               COMMON FUNCTIONS           *
 ********************************************/
function ejournal_delete_everything($jid, $iid) {
	$where = " jid='%d'";
	$arg[] = $jid;
	$affected = 0;

	if ($iid) {
		$where .= " AND iid='%d'";
		$arg[] = $iid;
	}

	$edit['confirm'] = 1;

	$result = db_query("SELECT nid FROM {ejournal_articles} WHERE $where", $arg);

	while ($nid = db_fetch_object($result)) {
		$edit['nid'] = $nid->nid;
		node_delete($edit); // this is CPU intensive
		$affected++;
	}

	return $affected;
}

// copied from flexinode_timestamp.inc
function _ejournal_date_form($timestamp = NULL, $prefix = '') {

	//determine settings for form's hour selector
	if (variable_get('event_ampm', '0')) {
		$hour_format = 'g';
		$first_hour = 1;
		$last_hour = 12;
	} else {
		$first_hour = 0;
		$last_hour = 23;
		$hour_format = 'G';
	}

	$years = drupal_map_assoc(range(date("Y", time()) - 10, date("Y", time()) + 1));
	$months = array (
		1 => t('January'
	), t('February'), t('March'), t('April'), t('May'), t('June'), t('July'), t('August'), t('September'), t('October'), t('November'), t('December'));
	for ($i = 1; $i <= 31; $i++)
		$days[$i] = $i;
	for ($i = $first_hour; $i <= $last_hour; $i++)
		$hours[$i] = $i;
	for ($i = 0; $i <= 59; $i++)
		$minutes[$i] = $i < 10 ? "0$i" : $i;
	$am_pms = array (
		'am' => t('am'
	), 'pm' => t('pm'));

	if (!isset ($timestamp)) {
		$timestamp = time();
		// Round to nearest hour:
		$timestamp -= $timestamp % (60 * 60);
	}

	// Use format_date(), it handles user timezone and locale.
	$year = format_date($timestamp, 'custom', 'Y');
	$month = format_date($timestamp, 'custom', 'm');
	$day = format_date($timestamp, 'custom', 'd');
	$hour = format_date($timestamp, 'custom', $hour_format);
	$minute = format_date($timestamp, 'custom', 'i');
	$am_pm = format_date($timestamp, 'custom', 'a');

	$when = array ();
	$when['day'] = array (
		'#type' => 'item',
		'#value' => '<h3>' . t("Issue published on"
	) . "</h3>",);
	$when['day'][$prefix . 'day'] = array (
		'#type' => 'select',
		'#default_value' => $day,
		'#options' => $days
	);
	$when['day'][$prefix . 'month'] = array (
		'#type' => 'select',
		'#default_value' => $month,
		'#options' => $months
	);
	$when['day'][$prefix . 'year'] = array (
		'#type' => 'select',
		'#default_value' => $year,
		'#options' => $years
	);
	$when['day'][$prefix . 'hour'] = array (
		'#type' => 'select',
		'#default_value' => $hour,
		'#options' => $hours
	);
	$when['day']['colon0'] = array (
		'#type' => 'item',
		'#value' => ':'
	);
	$when['day'][$prefix . 'minute'] = array (
		'#type' => 'select',
		'#default_value' => $minute,
		'#options' => $minutes
	);
	if (variable_get('event_ampm', '0')) {
		$when['day'][$prefix . 'ampm'] = array (
			'#type' => 'select',
			'#default_value' => $am_pm,
			'#options' => $am_pms
		);
	}

	return $when;
}

function _ejournal_validate_date(& $edit, $prefix = '') {

	if (isset ($edit[$prefix . 'year']) && isset ($edit[$prefix . 'month']) && isset ($edit[$prefix . 'day']) && isset ($edit[$prefix . 'hour']) && isset ($edit[$prefix . 'minute'])) {

		$hour = $edit[$prefix . 'hour'];
		if (variable_get('event_ampm', '0')) {
			if ($edit[$prefix . 'ampm'] == 'pm') {
				$hour += 12;
			} else
				if ($hour == 12) {
					$hour -= 12;
				}
		}
		$result = gmmktime($hour, $edit[$prefix . 'minute'], 0, $edit[$prefix . 'month'], $edit[$prefix . 'day'], $edit[$prefix . 'year']) - $GLOBALS['user']->timezone;
	} else if (isset ($edit[$prefix])) {
			$result = $edit[$prefix];
	}
	return $result;
}

function _delete_everything_form($type = 'issue', $selected_value = 0) {
	$delete = array (
		'0' => 'do not delete articles',
		'1' => 'delete all associated articles'
	);
	$ret = array ();

	return $ret['delete_everything'] = array (
		'#type' => 'select',
		'#title' => t("Delete everything?"), 
    '#default_value' => $selected_value, 
    '#options' => $delete, 
    '#description' => t("You may delete all documents that have been associated with this %type. But this cannot be undone! Note also, that deletion may fail if the current user does not have full rights to 'delete' all possible nodes.", array ('%type' => $type)),
    );

}

?>
